/*
Copyright (C) 2024 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');

const studentRevenueApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

const config = require(`${libs}/config`); 

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

studentRevenueApp.use(cache('15 day'));

studentRevenueApp.get('/years', (req, res, next) => {
    req.sql.from('pnad_novo')
    .field('DISTINCT receitas.ano', 'year')
    next();
}, query, response('years'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'min_year',
    table: 'receitas',
    tableField: 'ano',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano'
    }
}).addValue({
    name: 'max_year',
    table: 'receitas',
    tableField: 'ano',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano'
    }
}).addValue({
    name: 'city',
    table: 'receitas',
    tableField: 'cod_ibge',
    resultField: 'city',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_ibge'
    }
})

studentRevenueApp.get('/', rqf.parse(), rqf.build(),  (req, res, next) => {

    if (req.query.dims && req.query.dims.includes('receitas')) {
        req.sql.from('receitas')
        .field('receitas.ano', 'year')
        .field('receitas.receitas', 'total_receitas')
    }
    if (req.query.dims && req.query.dims.includes('matriculas_publicas')) {
        req.sql.from('receitas')
        .field('receitas.ano', 'year')
        .field('matriculas_publica', 'matriculas_publica_total')
    }
    if (req.query.dims && req.query.dims.includes('matriculas_publicas_mais_conveniada')) {
        req.sql.from('receitas')
        .field('receitas.ano', 'year')
        .field('matriculas_publicas_mais_conveniada', 'matriculas_publicas_mais_conveniada_total')
    }
    if (req.query.dims && req.query.dims.includes('receita_aluno_ano_publica')) {
        req.sql.from('receitas')
        .field('receitas.ano', 'year')
        .field('receuta_aluno_ano_publica', 'receita_aluno_ano_publica_total')
    }
    if (req.query.dims && req.query.dims.includes('receita_aluno_mes_publica')) {
        req.sql.from('receitas')
        .field('receitas.ano', 'year')
        .field('receita_aluno_mes_publica', 'receita_aluno_mes_publica_total')
    }
    if (req.query.dims && req.query.dims.includes('receita_aluno_ano_publica_mais_conveniada')) {
        req.sql.from('receitas')
        .field('receitas.ano', 'year')
        .field('receita_aluno_ano_publica_mais_conveniada', 'receita_aluno_ano_publica_mais_conveniada_total')
    }
    if (req.query.dims && req.query.dims.includes('receita_aluno_mes_publica_mais_conveniada')) {
        req.sql.from('receitas')
        .field('receitas.ano', 'year')
        .field('receita_aluno_mes_publica_mais_conveniada', 'receita_aluno_mes_publica_mais_conveniada_total')
    }

    next();
}, query, id2str.transform(false), response('student_revenue'));

module.exports = studentRevenueApp;
